/*
 * Created on Sep 11, 2004
 * Updated on Nov 23, 2005 by Majid
 */
package yaes.framework.entity.control.speedcontrol;

import yaes.framework.world.map.Location;
import yaes.framework.world.map.MapHelper;
import yaes.framework.world.map.PlannedPath;
import yaes.framework.world.map.Speed;
import yaes.ui.text.TextUi;

/**
 * This class provides a simple path follow control. The planned path and the
 * speed are provided. The given path is then followed at given speed.
 */
public class PathFollowControl extends AbstractSpeedControl {
	private final PlannedPath thePath;
	private final double speed; // in pixels per unit of time
	// Note that user can specify 0.2 pixels/time unit
	// which actually means to traverse one pixel in 5 time units
	private int lastCheckPoint; // last location used as check point
	private final double distanceEpsilon = 0.01; // error in distance
	private double updateTime = 0.0; // used to compute time elapsed since

	// last call
	public PathFollowControl(PlannedPath thePath, double speed,
			double updateTime) {
		this.thePath = thePath;
		this.speed = speed;
		this.updateTime = updateTime;
		lastCheckPoint = thePath.indexOfLocation(thePath.getSource());
	}

	// This method is called to control the controllable object
	@Override
	public void control(double time) {
		// Compute elapsed time
		final double deltaTime = time - updateTime;
		updateTime = time;
		if (deltaTime <= 0) {
			return;
		}
		if (controllable == null) {
			return;
		}
		if (thePath == null) {
			return;
		}
		if (this.speed <= 0) {
			controllable.controlSpeed(new Speed(0, 0));
			return;
		}
		// Compute the distance that need to be travelled (in pixels)
		final double distanceTravelled = speed * deltaTime;
		final Location currentLocation = controllable.getLocation();
		// Get the distance from destination location
		final double distance = MapHelper.distance(currentLocation, thePath
				.getDestination());
		// If destination has not reached
		if ((lastCheckPoint != thePath.getPathSize() - 1)
				|| (distance > distanceEpsilon)) {
			// TextUi.println("lastCheckPoint =" + lastCheckPoint);
			// TextUi.println("thePath.getPathSize() =" +
			// thePath.getPathSize());
			// Find the location that is distanceTravelled away from current
			// location on given path
			final Location nextLocation = findLocationAtDistance(
					currentLocation, distanceTravelled);
			// If such a location is found, use it to determine speed
			if (nextLocation != null) {
				final Speed speed = new Speed(
						(nextLocation.getX() - currentLocation.getX())
								/ deltaTime,
						(nextLocation.getY() - currentLocation.getY())
								/ deltaTime);
				controllable.controlSpeed(speed);
			}
		} else {
			TextUi.println("Destination reached!!!");
			controllable.controlSpeed(new Speed(0, 0));
		}
	}

	// This method is used to find a location on given path that is some
	// (distanceToTravel)
	// distance away from current location
	private Location findLocationAtDistance(Location currentLocation,
			double distanceToTravel) {
		Location loc = null;
		Location previousLocation = currentLocation;
		double dx = 0.0, dy = 0.0;
		double adjustedX = 0.0, adjustedY = 0.0;
		double distance = 0.0;
		double currentDistance = 0.0;
		// Starting from last check point, go through list of locations and
		// compute distance from current location to that location
		for (int i = lastCheckPoint; i < thePath.getPathSize(); i++) {
			loc = thePath.getLocationAt(i);
			currentDistance = MapHelper.distance(previousLocation, loc);
			distance += currentDistance;
			// If distance to a location is larger than distance to travel then
			// mark this location as next check point and use adjusted x,y
			// values to determine a middle location
			// that is exact distance to be travelled away from current location
			if (distance >= distanceToTravel) {
				lastCheckPoint = i;
				dx = loc.getX() - previousLocation.getX();
				dy = loc.getY() - previousLocation.getY();
				adjustedX = dx
						* (currentDistance - distance + distanceToTravel)
						/ currentDistance;
				adjustedY = dy
						* (currentDistance - distance + distanceToTravel)
						/ currentDistance;
				loc = new Location(previousLocation.getX() + adjustedX,
						previousLocation.getY() + adjustedY);
				return loc;
			} else if (i == thePath.getPathSize() - 1) {
				lastCheckPoint = i;
				return loc;
			}
			previousLocation = loc;
		}
		return null;
	}
}