package yaes.framework.simulation;

import java.io.Serializable;

import yaes.framework.simulation.statutil.StatProbe;
import yaes.framework.simulation.statutil.StudentDist;

/**
 * This class implements a random variable (almost the same thing as the SSJ
 * tally) but it is serializable and respects the Yaes call conventions.
 */
public class RandomVariable extends StatProbe implements Serializable {
	public enum Probe {
		AVERAGE, VALUE, LASTVALUE, COUNT, SUM, MAX, MIN, CONFINT_LOW, CONFINT_HIGH, CONFINT_RADIUS, VARIANCE
	}

	static final long serialVersionUID = 5130807822371431645L;;

	private int numObs;
	private double sumSquares;
	private double lastValue;

	public RandomVariable(String name) {
		this.name = name;
		init();
	}

	/**
	 * Required by the statistical probe interface
	 */
	@Override
	public double average() {
		return getAvg();
	}

	@Override
	public Object clone() {
		RandomVariable o = null;
		try {
			o = (RandomVariable) super.clone();
		} catch (final CloneNotSupportedException e) {
			System.err.println("Tally can't clone");
		}
		if (name != null) {
			o.name = new String(name);
		}
		return o;
	}

	public void confidenceIntervalStudent(double level,
			double centerAndRadius[]) {
		if (numObs < 2) {
			throw new RuntimeException(
					"Calling confidenceIntervalStudent with < 2 Observations");
		} else {
			centerAndRadius[0] = average();
			final double t = StudentDist.inverseF(numObs - 1,
					0.5D * (level + 1.0D));
			centerAndRadius[1] = t * Math.sqrt(getVariance() / numObs);
			return;
		}
	}

	/**
	 * Added by BL Equals function, value based
	 * 
	 * @return
	 */
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof RandomVariable)) {
			return false;
		}
		final RandomVariable other = (RandomVariable) o;
		// fall back to the equals of StatProbe
		if (!super.equals(o)) {
			return false;
		}
		if (numObs != other.numObs) {
			return false;
		}
		if (sumSquares != other.sumSquares) {
			return false;
		}
		return true;
	}

	/**
	 * @return Returns the average
	 */
	public double getAvg() {
		if (numObs > 0) {
			return sumValue / numObs;
		} else {
			return 0.0D;
		}
	}

	/**
	 * Return the 95% confidence interval as center and radius
	 * 
	 * @return
	 */
	public double[] getConfidenceInterval() {
		final double confint[] = new double[2];
		confidenceIntervalStudent(0.95, confint);
		return confint;
	}

	/**
	 * Return the confidence interval as center and radius
	 * 
	 * @return
	 */
	public double[] getConfidenceInterval(double percent) {
		final double confint[] = new double[2];
		confidenceIntervalStudent(percent, confint);
		return confint;
	}

	/**
	 * Return the number of observations recorder
	 * 
	 * @return
	 */
	public int getCount() {
		return numObs;
	}

	/**
	 * @return Returns the max.
	 */
	public double getMax() {
		return max();
	}

	/**
	 * @return Returns the min.
	 */
	public double getMin() {
		return min();
	}

	/**
	 * @return Returns the name.
	 */
	@Override
	public String getName() {
		return getName();
	}

	/**
	 * Returns a single value - but it throws a runtime exception if the value
	 * was updated more than ones
	 * 
	 * @return
	 */
	private double getSingleValue() {
		final int count = 0;
		if (count != 0) {
			throw new RuntimeException("Accessing a multiple times (" + count
					+ ") updated random variable " + name + " as single value!");
		}
		return getAvg();
	}

	/**
	 * Returns the standard deviation
	 * 
	 * @return
	 */
	public double getStandardDeviation() {
		return Math.sqrt(getVariance());
	}

	/**
	 * @return Returns the sum.
	 */
	public double getSum() {
		return sum();
	}

	public double getValue(Probe probe) {
		switch (probe) {
		case VALUE:
			return getSingleValue();
		case AVERAGE:
			return getAvg();
		case SUM:
			return getSum();
		case COUNT:
			return getCount();
		case MAX:
			return max();
		case MIN:
			return min();
		case CONFINT_LOW:
			return getConfidenceInterval()[0] - getConfidenceInterval()[1];
		case CONFINT_HIGH:
			return getConfidenceInterval()[0] + getConfidenceInterval()[1];
		case CONFINT_RADIUS:
			return getConfidenceInterval()[1];
		case VARIANCE:
			return getVariance();
		case LASTVALUE:
			return lastValue;
		}
		return 0;
	}

	/**
	 * Returns the variance on the random variable
	 * 
	 * @return
	 */
	public double getVariance() {
		if (numObs < 2) {
			throw new RuntimeException(
					"Calling variance with no enough Observations");
		}
		double temp = sumSquares - sumValue * sumValue / numObs;
		temp /= numObs - 1;
		if (temp < 0.0D) {
			return 0.0D;
		} else {
			return temp;
		}
	}

	@Override
	public void init() {
		maxValue = -1.0D / 0.0D;
		minValue = 1.0D / 0.0D;
		sumValue = 0.0D;
		sumSquares = 0.0D;
		numObs = 0;
		lastValue = 0.0D;
	}

	@Override
	public String report() {
		return toString();
	}

	/**
	 * Formatted print
	 */
	@Override
	public String toString() {
		if (getCount() == 1) {
			return name + " = " + getAvg() + "\n";
		}
		String temp = name + " = ";
		temp += getAvg() + " [" + getMin() + ".." + getMax() + "]";
		temp += "\tCount:\t" + getCount() + "\n";
		return temp;
	}

	/**
	 * Updates the random variable with a new observation
	 * 
	 * @param x
	 */
	public void update(double x) {
		lastValue = x;
		if (collect) {
			if (x < minValue) {
				minValue = x;
			}
			if (x > maxValue) {
				maxValue = x;
			}
			numObs++;
			sumValue += x;
			sumSquares += x * x;
		}
		if (broadcast) {
			setChanged();
			notifyObservers(new Double(x));
		}
	}
}
