package yaes.framework.simulation;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * @author Lotzi Boloni
 * 
 * This class contains the input of a simulation. It is a collection of
 * SimulationInputParameter objects, and the temporal dimension of the
 * simulation.
 * 
 */
public class SimulationInput implements Serializable {
	private static final long serialVersionUID = 10L;
	private double startTime = 0;
	private double stopTime = 1000;
	private double timeResolution = 1;
	private final HashMap<String, SimulationInputParameter> parameters = new HashMap<String, SimulationInputParameter>();

	/**
	 * Creates an empty simulation input object
	 * 
	 */
	public SimulationInput() {
		// nothing here
	}

	/**
	 * Creates a simulation input object initialized with the parameters of the
	 * model.
	 * 
	 * @param model
	 */
	public SimulationInput(SimulationInput model) {
		this.startTime = model.getStartTime();
		this.stopTime = model.getStopTime();
		this.timeResolution = model.getTimeResolution();
		for (final String name : model.parameters.keySet()) {
			parameters.put(name, model.parameters.get(name));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 * 
	 * Rewrite the equality for a value based one.
	 */
	@Override
	public boolean equals(Object o) {
		if (!(o instanceof SimulationInput)) {
			return false;
		}
		final SimulationInput other = (SimulationInput) o;
		if (startTime != other.startTime) {
			return false;
		}
		if (stopTime != other.stopTime) {
			return false;
		}
		if (timeResolution != other.timeResolution) {
			return false;
		}
		if (!parameters.equals(other.parameters)) {
			return false;
		}
		return true;
	}

	/**
	 * Returns the parameter
	 * 
	 * @param name
	 * @return
	 */
	public SimulationInputParameter getParameter(String name) {
		return parameters.get(name);
	}

	/**
	 * Returns a double valued parameter
	 * 
	 * @param name
	 * @return
	 */
	public double getParameterDouble(String name) {
		final SimulationInputParameter value = parameters.get(name);
		if (value == null) {
			throw new SimulationException("Cannot find parameter " + name);
		}
		return value.getDouble();
	}

	/**
	 * Returning an enumeration by the type passed.
	 * 
	 * @param <X>
	 * @param c
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public <X extends Enum> X getParameterEnum(Class<X> c) {
		final SimulationInputParameter value = parameters.get(c.getName());
		if (value == null) {
			throw new SimulationException("Cannot find parameter "
					+ c.getName());
		}
		return (X) value.getEnum();
	}

	/**
	 * Returns an integer valued parameter
	 * 
	 * @param name
	 * @return
	 */
	public int getParameterInteger(String name) {
		final SimulationInputParameter value = parameters.get(name);
		if (value == null) {
			throw new SimulationException("Cannot find parameter " + name);
		}
		return value.getInt();
	}

	/**
	 * Allows the iteration over the parameters (for external printing
	 * inspection etc)
	 * 
	 * @return
	 */
	public Iterator<String> getParameterIterator() {
		return parameters.keySet().iterator();
	}

	/**
	 * Returns a String valued parameter
	 * 
	 * @param name
	 * @return
	 */
	public String getParameterString(String name) {
		final SimulationInputParameter value = parameters.get(name);
		if (value == null) {
			throw new SimulationException("Cannot find parameter " + name);
		}
		return value.getString();
	}

	/**
	 * @return Returns the startTime.
	 */
	public double getStartTime() {
		return startTime;
	}

	/**
	 * @return Returns the stopTime.
	 */
	public double getStopTime() {
		return stopTime;
	}

	/**
	 * @return Returns the timeResolution.
	 */
	public double getTimeResolution() {
		return timeResolution;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 * 
	 * Rewritten to match with the equals
	 */
	@Override
	public int hashCode() {
		return parameters.hashCode();
	}

	/**
	 * Sets an enumeration valued parameter
	 * 
	 * @param e
	 */
	@SuppressWarnings("unchecked")
	public void setParameter(Enum e) {
		final SimulationInputParameter sim = new SimulationInputParameter(e);
		parameters.put(sim.getName(), sim);
	}

	/**
	 * Sets a double valued parameter
	 * 
	 * @param name
	 * @param value
	 */
	public void setParameter(String name, double value) {
		final SimulationInputParameter sim = new SimulationInputParameter(name,
				value);
		parameters.put(sim.getName(), sim);
	}

	/**
	 * Sets an integer valued parameter
	 * 
	 * @param name
	 * @param value
	 */
	public void setParameter(String name, int value) {
		final SimulationInputParameter sim = new SimulationInputParameter(name,
				value);
		parameters.put(sim.getName(), sim);
	}

	/**
	 * Sets a String valued parameter
	 * 
	 * @param name
	 * @param value
	 */
	public void setParameter(String name, SimulationInputParameter value) {
		parameters.put(name, value);
	}

	/**
	 * Sets a String valued parameter
	 * 
	 * @param name
	 * @param value
	 */
	public void setParameter(String name, String value) {
		final SimulationInputParameter sim = new SimulationInputParameter(name,
				value);
		parameters.put(sim.getName(), sim);
	}

	/**
	 * @param startTime
	 *            The startTime to set.
	 */
	public void setStartTime(double startTime) {
		this.startTime = startTime;
	}

	/**
	 * @param stopTime
	 *            The stopTime to set.
	 */
	public void setStopTime(double stopTime) {
		this.stopTime = stopTime;
	}

	/**
	 * @param timeResolution
	 *            The timeResolution to set.
	 */
	public void setTimeResolution(double timeResolution) {
		this.timeResolution = timeResolution;
	}

	/**
	 * Formatted printing of the object
	 */
	@Override
	public String toString() {
		final StringBuffer result = new StringBuffer("Input parameters ");
		result.append("simulation time: [" + startTime + ", " + stopTime + "]");
		final List<String> parlist = new ArrayList<String>();
		parlist.addAll(parameters.keySet());
		Collections.sort(parlist);
		for (final String key : parlist) {
			final SimulationInputParameter o = parameters.get(key);
			result.append("\n\t" + o.toString());
		}
		return result.toString();
	}
}
