package yaes.framework.world.sensornetwork;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import yaes.framework.agent.ACLMessage;
import yaes.framework.simulation.SimulationOutput;
import yaes.framework.world.World;
import yaes.framework.world.map.IObjectWithLocation;
import yaes.framework.world.sensornetwork.Perception.PerceptionType;

/**
 * This class manages the transmissions and perceptions of the agents
 * 
 * @author Lotzi Boloni
 * 
 */
public class SensorNetworkWorld extends World {
	public static final String HIDDEN_FIELD_SENDER_NODE = "senderNode";
	private final List<ActuatorNode> actuatorNodes = new ArrayList<ActuatorNode>();
	private final List<SensorNode> sensorNodes = new ArrayList<SensorNode>();
	private SinkNode sinkNode = new SinkNode();
	private final Map<IObjectWithLocation, SensingHistory> sensingHistory = new HashMap<IObjectWithLocation, SensingHistory>();
	private final ArrayList<ACLMessage> messageCache = new ArrayList<ACLMessage>();

	public SensorNetworkWorld(SimulationOutput so) {
		super(so);
	}

	public void addActuatorNode(ActuatorNode node) {
		actuatorNodes.add(node);
	}

	public void addSensorNode(SensorNode node) {
		sensorNodes.add(node);
	}

	/**
	 * Returns a read only list of actuator nodes
	 * 
	 * @return
	 */
	public List<ActuatorNode> getActuatorNodes() {
		return Collections.unmodifiableList(actuatorNodes);
	}

	/**
	 * Gets the sensing history associated with the sensor or actuator node (or
	 * creates it as needed)
	 * 
	 * @return
	 */
	public SensingHistory getSensingHistory(IObjectWithLocation node) {
		SensingHistory sh = sensingHistory.get(node);
		if (sh == null) {
			sh = new SensingHistory(this);
			sensingHistory.put(node, sh);
		}
		return sh;
	}

	/**
	 * Returns a read only list of sensor nodes
	 * 
	 * @return
	 */
	public List<SensorNode> getSensorNodes() {
		return Collections.unmodifiableList(sensorNodes);
	}

	public SinkNode getSinkNode() {
		return sinkNode;
	}

	/**
	 * Deliver all the pending messages
	 */
	public void messageFlush() {
		for (final ACLMessage message : messageCache) {
			transmitNow(message);
		}
		messageCache.clear();
	}

	/**
	 * An actuator node has moved. -it appears as an ActuatorPresence perception
	 * to the sensors whose sensing range it covers
	 * 
	 * @param node
	 */
	public void move(ActuatorNode actuatorNode) {
		for (final SensorNode node : sensorNodes) {
			final double distance = node.getLocation().distanceTo(
					actuatorNode.getLocation());
			if (distance <= node.getAgent().getSensorRange()) {
				final SensingHistory s = getSensingHistory(node);
				s.addPerception(new Perception(PerceptionType.ActuatorPresence,
						getTime()));
			}
		}
	}

	/**
	 * Processes the sensor nodes (including the sink)
	 * 
	 * @param sender
	 * @param message
	 */
	private void processSensorNode(SensorNode node, SensorNode sender,
			ACLMessage message) {
		final String destination = message.getDestination();
		boolean broadcast;
		if (destination.equals("*")) { // broadcast
			broadcast = true;
		} else {
			broadcast = false;
		}
		if (node == sender) {
			return;
		}
		final double distance = node.getLocation().distanceTo(
				sender.getLocation());
		if (distance <= sender.getAgent().getTransmissionRange()) {
			final SensingHistory s = getSensingHistory(node);
			PerceptionType pt;
			if (broadcast || destination.equals(node.getName())) {
				pt = PerceptionType.ReceivedMessage;
			} else {
				pt = PerceptionType.Overhearing;
			}
			s.addPerception(new Perception(pt, message, getTime()));
		}
	}

	public void setSinkNode(SinkNode sinkNode) {
		this.sinkNode = sinkNode;
	}

	/**
	 * 
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer("SensingManager\n");
		buffer
				.append("------------------ Actuator nodes ------------------------- ");
		return buffer.toString();
	}

	/**
	 * Transmission - but it will not be actually transmitted until the end of
	 * the cycle
	 * 
	 * @param message
	 */
	public void transmit(SensorNode node, ACLMessage message) {
		message.setValue(SensorNetworkWorld.HIDDEN_FIELD_SENDER_NODE, node);
		messageCache.add(message);
	}

	/**
	 * Transmits a message (generally defined message) -it will appear as a
	 * ReceivedMessage perception to other sensors in the transmission range -it
	 * will appear as an Overhearing perception to actuators in the transmission
	 * range
	 * 
	 * @param node
	 * @param message
	 */
	private void transmitNow(ACLMessage message) {
		getSimulationOutput().update("MessageCount", 1);
		final SensorNode sender = (SensorNode) message
				.getValue(SensorNetworkWorld.HIDDEN_FIELD_SENDER_NODE);
		// TextUi.println(message);
		// add the message to all the other sensors in the transmission range
		for (final SensorNode node : sensorNodes) {
			processSensorNode(node, sender, message);
		}
		processSensorNode(sinkNode, sender, message);
		// add the message to all the actuators in the transmission range as an
		// overhearing
		for (final ActuatorNode node : actuatorNodes) {
			final double distance = node.getLocation().distanceTo(
					sender.getLocation());
			if (distance < sender.getAgent().getTransmissionRange()) {
				final SensingHistory s = getSensingHistory(node);
				s.addPerception(new Perception(PerceptionType.Overhearing,
						message, getTime()));
			}
		}
	}
}
