package yaes.ui.propertysheet;

import java.awt.Component;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.ParseException;
import java.util.Vector;

import javax.swing.AbstractCellEditor;
import javax.swing.JFormattedTextField;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;

public class TableList extends JTable {
	class EmptyObject extends Object {
		@Override
		public String toString() {
			return "";
		}
	}

	class FormattedCellEditor extends AbstractCellEditor implements
			TableCellEditor {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private JFormattedTextField field;
		private final String className;

		public FormattedCellEditor(String className) {
			this.className = className;
			createFormattedField();
			field.addKeyListener(new KeyAdapter() {
				@Override
				public void keyPressed(KeyEvent e) {
					if (e.getKeyCode() == KeyEvent.VK_ENTER) {
						completeEdit();
						final String str = field.getValue().toString();
						field.setCaretPosition(str.length());
						FormattedCellEditor.this.fireEditingStopped();
					}
				}
			});
			field.addFocusListener(new FocusAdapter() {
				@Override
				public void focusLost(FocusEvent e) {
					completeEdit();
					FormattedCellEditor.this.fireEditingStopped();
				}
			});
		}

		public void completeEdit() {
			final int fb = field.getFocusLostBehavior();
			if ((fb == JFormattedTextField.COMMIT)
					|| (fb == JFormattedTextField.COMMIT_OR_REVERT)) {
				try {
					field.commitEdit();
					// Give it a chance to reformat.
					field.setValue(field.getValue());
				} catch (final ParseException pe) {
					if (fb == JFormattedTextField.COMMIT_OR_REVERT) {
						field.setValue(field.getValue());
					}
				}
			} else if (fb == JFormattedTextField.REVERT) {
				field.setValue(field.getValue());
			}
		}

		private void createFormattedField() {
			if (className.equals("java.lang.Integer")) {
				/*
				 * NumberFormat numberFormat = NumberFormat.getInstance();
				 * numberFormat.setParseIntegerOnly(true);
				 */
				final JFormattedTextField.AbstractFormatter intFormatter = new JFormattedTextField.AbstractFormatter() {
					/**
					 * 
					 */
					private static final long serialVersionUID = 1L;

					@Override
					public Object stringToValue(String text)
							throws ParseException {
						Integer value = null;
						try {
							value = Integer.valueOf(text);
						} catch (NumberFormatException exc) {
							throw new ParseException("Unparseable number.", 0);
						}
						return value;
					}

					@Override
					public String valueToString(Object value)
							throws ParseException {
						if (value == null) {
							throw new ParseException("Parse Error.", 0);
						}
						return value.toString();
					}
				};
				field = new JFormattedTextField(intFormatter);
			} else if (className.equals("java.lang.String")) {
				final JFormattedTextField.AbstractFormatter stringFormatter = new JFormattedTextField.AbstractFormatter() {
					/**
					 * 
					 */
					private static final long serialVersionUID = 1L;

					@Override
					public Object stringToValue(String text)
							throws ParseException {
						String value = "";
						if (text.length() > 0) {
							value = text.trim();
						}
						if (value.length() == 0) {
							throw new ParseException("Unparseable entry.", 0);
						}
						return value;
					}

					@Override
					public String valueToString(Object value)
							throws ParseException {
						if (value == null) {
							throw new ParseException("Parse Error.", 0);
						}
						return value.toString();
					}
				};
				field = new JFormattedTextField(stringFormatter);
			} else {
				field = new JFormattedTextField();
			}
		}

		public Object getCellEditorValue() {
			completeEdit();
			return field.getValue();
		}

		public Component getTableCellEditorComponent(JTable table,
				Object value, boolean isSelected, int row, int column) {
			field.setValue(table.getModel().getValueAt(row, column));
			return field;
		}
	}

	class ListDataModel extends AbstractTableModel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private Vector list;

		public ListDataModel(Vector list) {
			this.list = list;
		}

		public void add() {
			if (!list.isEmpty()
					&& (list.get(getRowCount() - 1) instanceof EmptyObject)) {
				errorPresenter.showError("Please enter data now.");
				return;
			}
			list.add(new EmptyObject());
			final int count = getRowCount();
			this.fireTableRowsInserted(count, count);
		}

		public int getColumnCount() {
			return 1;
		}

		public Vector getList() {
			return list;
		}

		public int getRowCount() {
			return list.size();
		}

		public Object getValueAt(int row, int column) {
			return list.elementAt(row);
		}

		@Override
		public boolean isCellEditable(int row, int column) {
			return true;
		}

		public void moveDown(int row) {
			if (list.isEmpty()) {
				return;
			}
			final Object obj = list.get(row);
			list.remove(row);
			list.insertElementAt(obj, row + 1);
			this.fireTableDataChanged();
		}

		public void moveUp(int row) {
			if (list.isEmpty()) {
				return;
			}
			final Object obj = list.get(row);
			list.remove(row);
			list.insertElementAt(obj, row - 1);
			this.fireTableDataChanged();
		}

		public void remove(int row) {
			if (list.isEmpty()) {
				return;
			}
			final int count = getRowCount();
			list.remove(row);
			this.fireTableRowsDeleted(count, count);
		}

		public void removeAll() {
			final int count = getRowCount();
			list.removeAllElements();
			this.fireTableRowsDeleted(0, count);
		}

		public void setList(Vector list) {
			this.list = list;
		}

		@Override
		public void setValueAt(Object value, int row, int column) {
			list.setElementAt(value, row);
			this.fireTableCellUpdated(row, column);
		}

		public void trim() {
			if (!list.isEmpty()) {
				final Object obj = list.get(getRowCount() - 1);
				if (obj instanceof EmptyObject) {
					list.remove(getRowCount() - 1);
				} else if (obj instanceof String) {
					final String str = (String) obj;
					if ((str.length() == 0) || (str.trim().length() == 0)) {
						list.remove(getRowCount() - 1);
					}
				}
			}
		}
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private final ListDataModel model;

	private ErrorPresenter errorPresenter;

	public TableList(Vector list) {
		model = new ListDataModel(list);
		this.setModel(model);
		this.setTableHeader(null);
		this.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setupEditor();
		if (this.getRowCount() > 0) {
			this.addRowSelectionInterval(0, 0);
		}
	}

	public void addRow() {
		model.add();
		this.setRowSelectionInterval(this.getRowCount() - 1,
				this.getRowCount() - 1);
	}

	public Vector getModifiedList() {
		return model.getList();
	}

	public void moveDown(int row) {
		if ((row == this.getRowCount() - 1) || (row == -1)) {
			return;
		}
		model.moveDown(row);
		this.addRowSelectionInterval(row + 1, row + 1);
	}

	public void moveUp(int row) {
		if ((row == 0) || (row == -1)) {
			return;
		}
		model.moveUp(row);
		this.addRowSelectionInterval(row - 1, row - 1);
	}

	@Override
	public void removeAll() {
		model.removeAll();
	}

	public void removeRow(int row) {
		model.remove(row);
	}

	public void setErrorPresenter(ErrorPresenter errorPresenter) {
		this.errorPresenter = errorPresenter;
	}

	public void setList(Vector list) {
		model.setList(list);
	}

	private void setupEditor() {
		final Vector list = model.getList();
		String className = "";
		if (list instanceof ValueList) {
			className = ((ValueList) list).getValueType();
		}
		final FormattedCellEditor editor = new FormattedCellEditor(className);
		this.getColumnModel().getColumn(0).setCellEditor(editor);
	}

	public void trim() {
		model.trim();
	}
}