package yaes.ui.simulationcontrol;

import javax.swing.JFrame;

import yaes.framework.simulation.IContext;
import yaes.framework.simulation.ISimulationCode;
import yaes.framework.simulation.SimulationInput;
import yaes.framework.simulation.SimulationOutput;
import yaes.ui.text.TextUi;
import yaes.ui.text.TextUiHelper;

/*
 * The simulation control framework.
 */
public class SimulationControlGui extends JFrame {
	public enum Command {
		NoCommand, Setup, Step, Run, Stop, PostProcess
	}

	public enum Status {
		BeforeSetup, ReadyToRun, Stopped, Running, BeforePostProcess, AfterPostProcess
	}

	private static final long serialVersionUID = -1268363099695354176L;

	/**
	 * Runs a single simulation run.
	 * 
	 * @param sim
	 * @param simulationCode
	 * @param context
	 * @return
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public static SimulationOutput simulate(SimulationInput sim,
			Class<? extends ISimulationCode> simulationCode, IContext context)
			throws InstantiationException, IllegalAccessException {
		final SimulationControlGui theGui = new SimulationControlGui();
		theGui.setVisible(true);
		theGui.runSimulation(sim, simulationCode, context);
		return theGui.sop;
	}

	private ISimulationCode code;

	private SimulationOutput sop = null;;

	private SimulationInput sin = null;

	private final SimulationControlPanel thePanel;
	private Command nextCommand;
	private Command currentCommand;
	private Status status;
	public double time = -1;

	private int runningDelay = 0;

	public SimulationControlGui() {
		super("YAES Simulation Control");
		currentCommand = Command.NoCommand;
		nextCommand = Command.NoCommand;
		status = Status.BeforeSetup;
		thePanel = new SimulationControlPanel(this);
		add(thePanel);
		pack();
	}

	/**
	 * It is used to check for the next command, without blocking the thread.
	 * 
	 * Returns true if there was a command.
	 */
	private boolean checkNextCommand() {
		if (nextCommand == Command.NoCommand) {
			return false;
		}
		currentCommand = nextCommand;
		nextCommand = Command.NoCommand;
		return true;
	}

	public int getRunningDelay() {
		return runningDelay;
	}

	public SimulationOutput getSimulationOutput() {
		return sop;
	}

	/**
	 * Returns a status string for visualization
	 */
	public String getStatusString() {
		if (sin == null) {
			return "" + status;
		} else {
			return status + ":" + time + "/" + sin.getStopTime();
		}
	}

	/**
	 * 
	 * This function runs the simulation, while waiting for control from the
	 * GUI.
	 * 
	 * @param sim
	 * @param simulationCode
	 * @param context
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 */
	public void runSimulation(SimulationInput sin,
			Class<? extends ISimulationCode> simulationCode, IContext context)
			throws InstantiationException, IllegalAccessException {
		TextUi.println("Initializing...");
		this.sin = sin;
		waitForCommand();
		long start = 0;
		if (currentCommand == Command.Setup) {
			code = simulationCode.newInstance();
			sop = new SimulationOutput(sin, context);
			start = System.currentTimeMillis();
			TextUi.println("Starting simulation: " + code);
			code.setup(sin, sop, context);
			currentCommand = Command.Stop;
			status = Status.ReadyToRun;
		} else {
			throw new Error("Invalid command " + currentCommand);
		}
		// ready to run, what for a Step or Run to run
		waitForCommand();
		if ((currentCommand == Command.Run) || (currentCommand == Command.Step)) {
			status = Status.Running;
		} else {
			throw new Error("Invalid command " + currentCommand);
		}
		for (time = sin.getStartTime(); time < sin.getStopTime(); time = time
				+ sin.getTimeResolution()) {
			if (currentCommand == Command.Stop) {
				waitForCommand();
			}
			// run one step
			if (code.update(time, sin, sop, context) == 0) {
				break;
			}
			if (currentCommand == Command.Step) {
				currentCommand = Command.Stop;
			}
			if (currentCommand == Command.Run) {
				checkNextCommand();
			}
			// FIXME: this needs to be more complicated, in the sense of
			// maintaining a smooth run, independently on the simulation time.
			if (runningDelay != 0) {
				try {
					Thread.sleep(runningDelay);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			thePanel.update();
		}
		TextUi.println("Simulation run done, postprocessing.");
		code.postprocess(sin, sop, context);
		TextUi.println("Done. This simulation step took: "
				+ TextUiHelper.formatTimeInterval(System.currentTimeMillis()
						- start));
		TextUi.println("Memory: " + Runtime.getRuntime().freeMemory() / 1024
				+ "KB free / " + Runtime.getRuntime().totalMemory() / 1024
				+ "KB total");
	}

	/**
	 * This function is used by the panel and other utilities to set the next
	 * command.
	 * 
	 * @param command
	 */
	synchronized public void setNextCommand(Command command) {
		nextCommand = command;
		this.notifyAll();
	}

	public void setRunningDelay(int runningDelay) {
		this.runningDelay = runningDelay;
	}

	/**
	 * Stops the simulation and waits for the next command, by waiting on the
	 * thread. It is waken up by the setNextCommand function.
	 */
	private synchronized void waitForCommand() {
		while (true) {
			if (nextCommand != Command.NoCommand) {
				currentCommand = nextCommand;
				nextCommand = Command.NoCommand;
				return;
			}
			try {
				// synchronized (this) {
				this.wait();
				// }
				// Thread.currentThread().sleep(100);
			} catch (final InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}
