package yaes.ui.visualization.application.sensornetwork;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.util.List;

import yaes.framework.agent.ACLMessage;
import yaes.framework.world.map.Location;
import yaes.framework.world.sensornetwork.AbstractSensorAgent;
import yaes.framework.world.sensornetwork.ActuatorAgent;
import yaes.framework.world.sensornetwork.ActuatorNode;
import yaes.framework.world.sensornetwork.Perception;
import yaes.framework.world.sensornetwork.SensorNetworkWorld;
import yaes.framework.world.sensornetwork.SensorNode;
import yaes.framework.world.sensornetwork.SinkNode;
import yaes.ui.visualization.IPainter;
import yaes.ui.visualization.VisualMapPanel;
import yaes.util.DrawArrow;

/**
 * Paints the communication and sensing acts of the nodes
 * 
 * @author Lotzi Boloni <lboloni@eecs.ucf.edu>
 * 
 */
public class SensorNetworkWorldPainter implements IPainter {
	PainterConfiguration painterConf;

	public SensorNetworkWorldPainter(PainterConfiguration painterConf) {
		this.painterConf = painterConf;
	}

	/**
	 * Painting: basically, paint the perceptions for everybody
	 */
	public void paint(Graphics2D g, Object o, VisualMapPanel panel) {
		final SensorNetworkWorld sensorWorld = (SensorNetworkWorld) o;
		for (final SensorNode sensorNode : sensorWorld.getSensorNodes()) {
			paintSensorPerceptions(g, sensorNode, sensorWorld);
		}
		final SinkNode sinkNode = sensorWorld.getSinkNode();
		paintSensorPerceptions(g, sinkNode, sensorWorld);
		for (final ActuatorNode actuatorNode : sensorWorld.getActuatorNodes()) {
			paintActuatorPerceptions(g, panel, actuatorNode, sensorWorld);
		}
	}

	/**
	 * Paints the perceptions of the actuator
	 * @param g
	 * @param node
	 * @param sensorWorld
	 */
	private void paintActuatorPerceptions(Graphics2D g, VisualMapPanel panel, ActuatorNode node,
			SensorNetworkWorld sensorWorld) {
		final Point2D.Double point = node.getLocation().getRoundedLocation().asPoint();
		panel.getTheTransform().transform(point, point);
		final int x = (int) point.getX();
		final int y = (int) point.getY();
		final ActuatorAgent agent = (ActuatorAgent) node.getAgent();
		final List<Perception> perceptions =
				sensorWorld.getSensingHistory(node).inspectLastPerceptions(
						(Integer)painterConf.get(PainterConfiguration.PCONF_SECONDS_TO_CONSIDER));
		for (final Perception p : perceptions) {
			switch (p.getType()) {
			case ReceivedMessage: {
				// draw a line to the node which sent the message
				final ACLMessage m = p.getMessage();
				//int intensity = (Integer)m.getValue(MessageConstants.FIELD_INTENSITY);
				//if (intensity == 0) { 
				//	g.setColor(Color.yellow);
				//} else {
					g.setColor(Color.black);
				//}
				final SensorNode sender =
						(SensorNode) m
								.getValue(SensorNetworkWorld.HIDDEN_FIELD_SENDER_NODE);
				final Point2D.Double senderPoint =
						sender.getLocation().getRoundedLocation().asPoint();
				panel.getTheTransform().transform(senderPoint, senderPoint);
				final float stroke = 1.0f;
				DrawArrow.drawArrow(g, (int) senderPoint.getX(),
						(int) senderPoint.getY(), x, y, stroke);
				break;
			}
			}
		}
	}

	/**
	 * Paints the perceptions
	 * 
	 * @param g
	 * @param agent
	 */
	private void paintSensorPerceptions(Graphics2D g, SensorNode node,
			SensorNetworkWorld sensorWorld) {
		final Location loc = node.getLocation().getRoundedLocation();
		final AbstractSensorAgent agent = node.getAgent();
		final int x = (int) loc.getX();
		final int y = (int) loc.getY();
		final List<Perception> perceptions =
				sensorWorld.getSensingHistory(node).inspectLastPerceptions(
						(Integer)painterConf.get(PainterConfiguration.PCONF_SECONDS_TO_CONSIDER));
		for (final Perception p : perceptions) {
			switch (p.getType()) {
			case ActuatorPresence: {
				// blink the sensor range red.
				g.setColor((Color)painterConf.get(PainterConfiguration.PCONF_SENSOR_RANGE_COLOR));
				final Composite save = g.getComposite();
				g.setComposite(AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, (float) 0.2));
				g.fillOval((int) (x - agent.getSensorRange()), (int) (y - agent
						.getSensorRange()), (int) (2 * agent.getSensorRange()),
						(int) (2 * agent.getSensorRange()));
				g.setComposite(save);
				break;
			}
			case ReceivedMessage: {
				// draw a line to the node which sent the message
				final ACLMessage m = p.getMessage();
				/*
				String content = (String) m.getValue(MessageConstants.FIELD_CONTENT);
				if (content.equals(MessageConstants.MESSAGE_INTEREST)) {
					g.setColor(Color.black);
				}
				if (content.equals(MessageConstants.MESSAGE_DATA)) {
					int intensity = (Integer)m.getValue(MessageConstants.FIELD_INTENSITY);					
					if (intensity == 0) { 
						g.setColor(Color.yellow);
					} else {
						g.setColor(Color.red);
					}					
				}	
				*/									
				g.setColor(Color.black);
				final SensorNode sender =
						(SensorNode) m
								.getValue(SensorNetworkWorld.HIDDEN_FIELD_SENDER_NODE);
				final Location senderLocation =
						sender.getLocation().getRoundedLocation();
				final float stroke = 1.0f;
				DrawArrow.drawArrow(g, (int) senderLocation.getX(),
						(int) senderLocation.getY(), x, y, stroke);
				break;
			}
			}
		}
	}

	/**
	 * The layer is Background + 1
	 */
	public int getLayer() {
		return VisualMapPanel.BACKGROUND_LAYER - 1;
	}
}
